/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dubbothrift

import (
	"bytes"

	"mosn.io/mosn/pkg/protocol/xprotocol"
	"mosn.io/mosn/pkg/types"
)

func init() {
	//thrift must match before tars
	xprotocol.RegisterMatcher(ProtocolName, thriftMatcher)
}

// predicate thrift header len and compare magic number
func thriftMatcher(data []byte) types.MatchResult {
	//4 byte message length + 2 byte magic
	if len(data) < MessageLenSize+MagicLen {
		return types.MatchAgain
	}
	if bytes.Compare(data[MessageLenSize:MessageLenSize+MagicLen], MagicTag) != 0 {
		return types.MatchFailed
	}
	return types.MatchSuccess
}
